/**
  Sample SourceCode For NE-USB

  @Company
	Patlite Corporation

  @File Name
	patlite.c

  @Summary
	This file defines the functions to control NE-USB.

  @Description
	It describes how to turn on the LED and blow the buzzer of NE-USB.

*/
#include "hidapi_linux/hidapi.h"	// for linux
// #include "hidapi_mac/hidapi.h"		// for macOS
#include <stdint.h>
#include <stdio.h>
#include <stdbool.h>
#include "patlite.h"

#define PATLITE_VID (0x191A)
#define PATLITE_PID (0x6001)

#define COMMAND_SIZE 9

static hid_device *patlite_handle;

int patlite_lights(LED_COLORS color, LED_PATTERNS ptn)
{
	uint8_t buf[COMMAND_SIZE] = {0x00, 0x00, 0x00, 0x08, 0x0f, 0x00, 0x00, 0x00, 0x00};
	int led_pos = 5;

	switch (color) {
		case LED_COLOR_OFF:
			buf[led_pos] = 0x00;
			break;
		case LED_COLOR_RED:
			buf[led_pos] = 0x10;
			break;
		case LED_COLOR_GREEN:
			buf[led_pos] = 0x20;
			break;
		case LED_COLOR_YELLOW:
			buf[led_pos] = 0x30;
			break;
		case LED_COLOR_BLUE:
			buf[led_pos] = 0x40;
			break;
		case LED_COLOR_PURPLE:
			buf[led_pos] = 0x50;
			break;
		case LED_COLOR_CYAN:
			buf[led_pos] = 0x60;
			break;
		case LED_COLOR_WHITE:
			buf[led_pos] = 0x70;
			break;
		default:
			buf[led_pos] = 0xF0;
			break;
	}

	switch (ptn) {
		case LED_PATTERN_OFF:
			buf[led_pos] |= 0x00;
			break;
		case LED_PATTERN_CONTINUOUS:
			buf[led_pos] |= 0x01;
			break;
		case LED_PATTERN_NO1:
			buf[led_pos] |= 0x02;
			break;
		case LED_PATTERN_NO2:
			buf[led_pos] |= 0x03;
			break;
		case LED_PATTERN_NO3:
			buf[led_pos] |= 0x04;
			break;
		case LED_PATTERN_NO4:
			buf[led_pos] |= 0x05;
			break;
		case LED_PATTERN_NO5:
			buf[led_pos] |= 0x06;
			break;
		case LED_PATTERN_NO6:
			buf[led_pos] |= 0x07;
			break;
		default:
			buf[led_pos] |= 0x0F;
			break;
	}
	return patlite_set(buf);
}

int patlite_buzzer(BUZZER_PATTERNS ptn)
{
	uint8_t buf[COMMAND_SIZE] = {0x00, 0x00, 0x00, 0x00, 0x0A, 0xff, 0x00, 0x00, 0x00};
	int buz_pos = 3;

	switch (ptn) {
		case BUZZER_PATTERN_OFF:
			buf[buz_pos] |= 0x00;
			break;
		case BUZZER_PATTERN_CONTINUOUS:
			buf[buz_pos] |= 0x01;
			break;
		case BUZZER_PATTERN_NO1:
			buf[buz_pos] |= 0x02;
			break;
		case BUZZER_PATTERN_NO2:
			buf[buz_pos] |= 0x03;
			break;
		case BUZZER_PATTERN_NO3:
			buf[buz_pos] |= 0x04;
			break;
		case BUZZER_PATTERN_NO4:
			buf[buz_pos] |= 0x05;
			break;
		case BUZZER_PATTERN_NO5:
			buf[buz_pos] |= 0x06;
			break;
		case BUZZER_PATTERN_NO6:
			buf[buz_pos] |= 0x07;
			break;
		default:
			buf[buz_pos] |= 0x0f;
			break;
	}

	return patlite_set(buf);
}

int patlite_set(uint8_t *buf)
{
	int result = 0;

	if (patlite_handle == NULL) {
		fprintf(stderr, "Unable to open device with product ID %i. Please check that it is connected.\n", PATLITE_PID);
		hid_exit();
		result = -1;
	} else {
		result = hid_write(patlite_handle, buf, COMMAND_SIZE);

		// Was the write successful?
		if (result == -1) {
			fprintf(stderr, "Patlite set failed, return %d, the handle will be destroyed.\n", result);
		}
	}

	// printf("Patlite set return %d.\n", result);
	return result;
}

int patlite_device_open(void)
{
	int result = 0;

	// Open the device using the VID, PID, and optionally the Serial number.
	patlite_handle = hid_open(PATLITE_VID, PATLITE_PID, NULL);

	if (patlite_handle != NULL) {
		fprintf(stdout, "Succeeded to open device with product ID %i.\n", PATLITE_PID);
	} else {
		fprintf(stderr, "Unable to open device with product ID %i. Please check that it is connected.\n", PATLITE_PID);
		hid_exit();
		result = -1;
	}
	return result;
}

int patlite_device_close(void)
{
	int result = 0;

	hid_close(patlite_handle);
	fprintf(stdout, "Succeeded to close device with product ID %i.\n", PATLITE_PID);
	return result;
}

int patlite_init()
{
	hid_init();
	return 0;
}
